class Patent
  include Mongoid::Document
  include Mongoid::Timestamps
  include OrbitModel::Status
  include Slug

  has_and_belongs_to_many :patent_types
  has_and_belongs_to_many :patent_author_types
  belongs_to :member_profile

  field :patent_title, as: :slug_title, localize: true
  field :authors, localize: true
  field :patent_country, localize: true
  field :patent_organization, localize: true

  field :year
  field :language
  field :keywords
  field :patent_no
  field :progress_status
  field :application_date, type: DateTime
  field :end_date, type: DateTime
  field :publish_date , :type => Date
  field :url
  field :note
  field :rss2_id
  field :create_user_id, :type => BSON::ObjectId
  field :update_user_id, :type => BSON::ObjectId

  # paginates_per 10
  
  has_many :patent_files, :autosave => true, :dependent => :destroy
  accepts_nested_attributes_for :patent_files, :allow_destroy => true

  before_validation :add_http

  scope :sort_for_frontend, ->{ where(:is_hidden=>false).order_by(:year => "desc", :application_date => "desc", :publish_date => "desc") }

  def get_plugin_data(fields_to_show)
    plugin_datas = []
    fields_to_show.each do |field|
      plugin_data = self.get_plugin_field_data(field) rescue nil
      next if plugin_data.blank? or plugin_data['value'].blank?
      plugin_datas << plugin_data
    end
    plugin_datas
  end

  def self.get_plugin_datas_to_member(datas)
    page = Page.where(:module => "personal_patent").first rescue nil
    
    if !page.nil? && !page.custom_array_field.blank?
      fields_to_show = page.custom_array_field
    else
      fields_to_show = [
        "publish_date",
        "patent_title",
        "patent_no",
        "patent_country",
        "authors"
      ]
    end

    fields_to_remove = []

    pd_title = []

    fields_to_show.each do |t|
      if (self.fields[t].type.to_s == "String" rescue false)
        fields_to_remove << t if (datas.where(t.to_sym.ne => nil).or(t.to_sym.ne => "").count == 0 rescue false)
      else
        t = "patent_types" if t == "patent_category"
        t = "patent_author_types" if t == "author_type"
        fields_to_remove << t if (datas.where(t.to_sym.ne => nil).count == 0 rescue false)
        t = "patent_category" if t == "patent_types"
        t = "author_type" if t == "patent_author_types"
      end
      pd_title <<  {
          "plugin_data_title" => I18n.t("personal_patent.#{t}")
        } if !fields_to_remove.include?(t)
    end

    fields_to_show = fields_to_show - fields_to_remove

    plugin_datas = datas.sort_for_frontend.collect do |p|

      pd_data = []
      fields_to_show.collect do |t|
         case t
          when "patent_title"
            pd_data << {"data_title" => "<a href='#{OrbitHelper.url_to_plugin_show(p.to_param,'personal_patent')}'  title='" + (p.send(t) rescue "") +"'  target='_blank'>" + (p.send(t) rescue "") + "</a>"}
          when "publish_date"
            pd_data << {"data_title" => (p.publish_date.strftime("%Y/%m/%d") rescue "")}
          when "application_date"
            pd_data << {"data_title" => (p.application_date.strftime("%Y/%m/%d") rescue "")}
          when "end_date"
            pd_data << {"data_title" => (p.end_date.strftime("%Y/%m/%d") rescue "")}
          when "patent_category"
            pd_data << {"data_title" => (p.patent_types.collect{|pt| pt.title}.join(", ") rescue "")}
          when "author_type"
            pd_data << {"data_title" => (p.patent_author_types.collect{|pt| pt.title}.join(", ") rescue "")}
          else
            pd_data << {"data_title" => p.send(t)}
        end
      end

      {
        "pd_datas" => pd_data
      }

    end

    return [pd_title,plugin_datas]

  end

  def get_plugin_field_data(field)
    case field
    when "patent_category"
      value = self.patent_types.collect{|patent_type| patent_type.title}.join(',') rescue ""
    when "language"
       value = I18n.t(self.language) if !self.language.nil?  rescue ""
    when "publish_date"
      value =  self.publish_date.strftime("%Y/%m/%d") rescue ""
    when "application_date"
      value =  self.application_date.strftime("%Y/%m/%d") rescue ""
    when "end_date"
      value =  self.end_date.strftime("%Y/%m/%d") rescue ""
    when "author_type"
      value =  self.patent_author_types.collect{|pt| pt.title}.join(", ") rescue ""
    when "file"
      files = []
      self.patent_files.each do |patent_file|  
        url = patent_file.file.url
        title = ((patent_file.title.blank? ? File.basename(patent_file.file.path) : patent_file.title) rescue "")
        files << (url.nil? ? "" : "<li><a href='#{url}'' target='_blank'>#{title}</li>")
      end
      value = files.join("")
    else
      value = self.send(field) rescue ""
    end

    value = (value =~ /\A#{URI::regexp(['http', 'https'])}\z/) ? "<a href='#{value}' target='_blank'>#{value}</a>" : value
    
    {
      "key"=>field,
      "title_class"=>"patent-#{field.gsub('_','-')}-field",
      "value_class"=>"patent-#{field.gsub('_','-')}-value",
      "title"=>I18n.t('personal_patent.'+field),
      "value"=>value
    }
  end

  protected

  def add_http
    unless self.url.blank? || self.url[/^http:\/\//] || self.url[/^https:\/\//]
      self.url = 'http://' + self.url
    end
  end
  
end